;
;  Port on Texas Instruments TMS320C6x architecture
;
;  Copyright (C) 2004-2011 Texas Instruments Incorporated
;  Author: Aurelien Jacquiot (aurelien.jacquiot@virtuallogix.com)
;  Updated for 2.6.34: Mark Salter <msalter@redhat.com>
;
;  This program is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License version 2 as
;  published by the Free Software Foundation.
;

#include <linkage.h>

; Registers naming
#define DP	B14
#define SP	B15

	.altmacro

	.macro MASK_INT reg
	MVC	.S2	CSR,reg
	CLR	.S2	reg,0,0,reg
	MVC	.S2	reg,CSR
	.endm

	.macro UNMASK_INT reg
	MVC	.S2	CSR,reg
	SET	.S2	reg,0,0,reg
	MVC	.S2	reg,CSR
	.endm

	;;
	;;  This defines the normal kernel pt_regs layout.
	;;
	.macro SAVE_ALL __rp __tsr
	STDW	.D2T2	SP:DP,*--SP[1]
	SUBAW	.D2	SP,2,SP

	ADD	.D1X	SP,-8,A15
 ||	STDW	.D2T1	A15:A14,*SP--[16]

	STDW	.D2T2	B13:B12,*SP--[1]
 ||	STDW	.D1T1	A13:A12,*A15--[1]
 ||	MVC	.S2	__rp,B13

	STDW	.D2T2	B11:B10,*SP--[1]
 ||	STDW	.D1T1	A11:A10,*A15--[1]
 ||	MVC	.S2	CSR,B12

	STDW	.D2T2	B9:B8,*SP--[1]
 ||	STDW	.D1T1	A9:A8,*A15--[1]
 ||	MVC	.S2	RILC,B11
	STDW	.D2T2	B7:B6,*SP--[1]
 ||	STDW	.D1T1	A7:A6,*A15--[1]
 ||	MVC	.S2	ILC,B10

	STDW	.D2T2	B5:B4,*SP--[1]
 ||	STDW	.D1T1	A5:A4,*A15--[1]

	STDW	.D2T2	B3:B2,*SP--[1]
 ||	STDW	.D1T1	A3:A2,*A15--[1]
 ||	MVC	.S2	__tsr,B5

	STDW	.D2T2	B1:B0,*SP--[1]
 ||	STDW	.D1T1	A1:A0,*A15--[1]
 ||	MV	.S1X	B5,A5

	STDW	.D2T2	B31:B30,*SP--[1]
 ||	STDW	.D1T1	A31:A30,*A15--[1]
	STDW	.D2T2	B29:B28,*SP--[1]
 ||	STDW	.D1T1	A29:A28,*A15--[1]
	STDW	.D2T2	B27:B26,*SP--[1]
 ||	STDW	.D1T1	A27:A26,*A15--[1]
	STDW	.D2T2	B25:B24,*SP--[1]
 ||	STDW	.D1T1	A25:A24,*A15--[1]
	STDW	.D2T2	B23:B22,*SP--[1]
 ||	STDW	.D1T1	A23:A22,*A15--[1]
	STDW	.D2T2	B21:B20,*SP--[1]
 ||	STDW	.D1T1	A21:A20,*A15--[1]
	STDW	.D2T2	B19:B18,*SP--[1]
 ||	STDW	.D1T1	A19:A18,*A15--[1]
	STDW	.D2T2	B17:B16,*SP--[1]
 ||	STDW	.D1T1	A17:A16,*A15--[1]

	STDW	.D2T2	B13:B12,*SP--[1]	; save PC and CSR

	STDW	.D2T2	B11:B10,*SP--[1]	; save RILC and ILC
	STDW	.D2T1	A5:A4,*SP--[1]		; save TSR and orig A4
	.endm

	.macro RESTORE_ALL __rp __tsr
	LDDW	.D2T2	*++SP[1],B9:B8		; get TSR (B9)
	LDDW	.D2T2	*++SP[1],B11:B10	; get RILC (B11) and ILC (B10)
	LDDW	.D2T2	*++SP[1],B13:B12	; get PC (B13) and CSR (B12)

	ADDAW	.D1X	SP,30,A15

	LDDW	.D1T1	*++A15[1],A17:A16
 ||	LDDW	.D2T2	*++SP[1],B17:B16
	LDDW	.D1T1	*++A15[1],A19:A18
 ||	LDDW	.D2T2	*++SP[1],B19:B18
	LDDW	.D1T1	*++A15[1],A21:A20
 ||	LDDW	.D2T2	*++SP[1],B21:B20
	LDDW	.D1T1	*++A15[1],A23:A22
 ||	LDDW	.D2T2	*++SP[1],B23:B22
	LDDW	.D1T1	*++A15[1],A25:A24
 ||	LDDW	.D2T2	*++SP[1],B25:B24
	LDDW	.D1T1	*++A15[1],A27:A26
 ||	LDDW	.D2T2	*++SP[1],B27:B26
	LDDW	.D1T1	*++A15[1],A29:A28
 ||	LDDW	.D2T2	*++SP[1],B29:B28
	LDDW	.D1T1	*++A15[1],A31:A30
 ||	LDDW	.D2T2	*++SP[1],B31:B30

	LDDW	.D1T1	*++A15[1],A1:A0
 ||	LDDW	.D2T2	*++SP[1],B1:B0

	LDDW	.D1T1	*++A15[1],A3:A2
 ||	LDDW	.D2T2	*++SP[1],B3:B2
 ||	MVC	.S2	B9,__tsr
	LDDW	.D1T1	*++A15[1],A5:A4
 ||	LDDW	.D2T2	*++SP[1],B5:B4
 ||	MVC	.S2	B11,RILC
	LDDW	.D1T1	*++A15[1],A7:A6
 ||	LDDW	.D2T2	*++SP[1],B7:B6
 ||	MVC	.S2	B10,ILC

	LDDW	.D1T1	*++A15[1],A9:A8
 ||	LDDW	.D2T2	*++SP[1],B9:B8
 ||	MVC	.S2	B13,__rp

	LDDW	.D1T1	*++A15[1],A11:A10
 ||	LDDW	.D2T2	*++SP[1],B11:B10
 ||	MVC	.S2	B12,CSR

	LDDW	.D1T1	*++A15[1],A13:A12
 ||	LDDW	.D2T2	*++SP[1],B13:B12

	MV	.D2X	A15,SP
	LDDW	.D2T1	*++SP[1],A15:A14

#if 0
	NOP	5

	MVKL	.S2	0xc3f00000,B0
	MVKH	.S2	0xc3f00000,B0
	MVKL	.S1	0xc3f00100,A0
	MVKH	.S1	0xc3f00100,A0
	STW	.D2T2	B0,*B0++
	STW	.D2T2	B1,*B0++
	STW	.D2T2	B2,*B0++
	STW	.D2T2	B3,*B0++
	STW	.D2T2	B4,*B0++
	STW	.D2T2	B5,*B0++
	STW	.D2T2	B6,*B0++
	STW	.D2T2	B7,*B0++
	STW	.D2T2	B8,*B0++
	STW	.D2T2	B9,*B0++
	STW	.D2T2	B10,*B0++
	STW	.D2T2	B11,*B0++
	STW	.D2T2	B12,*B0++
	STW	.D2T2	B13,*B0++
	STW	.D2T2	B14,*B0++
	STW	.D2T2	B15,*B0++
	STW	.D2T2	B16,*B0++
	STW	.D2T2	B17,*B0++
	STW	.D2T2	B18,*B0++
	STW	.D2T2	B19,*B0++
	STW	.D2T2	B20,*B0++
	STW	.D2T2	B21,*B0++
	STW	.D2T2	B22,*B0++
	STW	.D2T2	B23,*B0++
	STW	.D2T2	B24,*B0++
	STW	.D2T2	B25,*B0++
	STW	.D2T2	B26,*B0++
	STW	.D2T2	B27,*B0++
	STW	.D2T2	B28,*B0++
	STW	.D2T2	B29,*B0++
	STW	.D2T2	B30,*B0++
	STW	.D2T2	B31,*B0++
	
	STW	.D1T1	A0,*A0++
	STW	.D1T1	A1,*A0++
	STW	.D1T1	A2,*A0++
	STW	.D1T1	A3,*A0++
	STW	.D1T1	A4,*A0++
	STW	.D1T1	A5,*A0++
	STW	.D1T1	A6,*A0++
	STW	.D1T1	A7,*A0++
	STW	.D1T1	A8,*A0++
	STW	.D1T1	A9,*A0++
	STW	.D1T1	A10,*A0++
	STW	.D1T1	A11,*A0++
	STW	.D1T1	A12,*A0++
	STW	.D1T1	A13,*A0++
	STW	.D1T1	A14,*A0++
	STW	.D1T1	A15,*A0++
	STW	.D1T1	A16,*A0++
	STW	.D1T1	A17,*A0++
	STW	.D1T1	A18,*A0++
	STW	.D1T1	A19,*A0++
	STW	.D1T1	A20,*A0++
	STW	.D1T1	A21,*A0++
	STW	.D1T1	A22,*A0++
	STW	.D1T1	A23,*A0++
	STW	.D1T1	A24,*A0++
	STW	.D1T1	A25,*A0++
	STW	.D1T1	A26,*A0++
	STW	.D1T1	A27,*A0++
	STW	.D1T1	A28,*A0++
	STW	.D1T1	A29,*A0++
	STW	.D1T1	A30,*A0++
	STW	.D1T1	A31,*A0++

	LDDW	.D2T2	*+SP[3],SP:DP
	NOP	5
	
	STW	.D2T2	SP,*B0++
	STW	.D2T2	DP,*B0++

foo:
	B	.S2	foo
	NOP	5
#endif

	B	.S2	__rp		; return from interruption
	LDDW	.D2T2	*+SP[1],SP:DP
	NOP	4
	.endm

	.section .text
ENTRY(system_call_saved)
	MVK	.L2	1,B2
	MVC	.S2	B2,ECR			; ack the software exception
	UNMASK_INT B2			; re-enable global IT
	B	.S2	system_call
	ADDKPC	.S2	ret_from_syscall_function,B3,4

ret_from_syscall_function:
	MASK_INT B2

	;;
	;; On C64x+, the return way from exception and interrupt
	;; is a little bit different
	;;
ENTRY(ret_from_exception)

ENTRY(ret_from_interrupt)
restore_all:
	RESTORE_ALL NRP,NTSR

	;; These are the interrupt handlers, responsible for calling c6x_do_IRQ()
	;;
	.macro SAVE_ALL_INT
	SAVE_ALL IRP,ITSR
	.endm

	.macro CALL_INT int
	CALLP   .S2	c6x_do_IRQ,B3
 ||	MVK	.S1	int,A4
 ||	ADDAW	.D2	SP,2,B4
	B	.S1	ret_from_interrupt
	NOP	5
	.endm

ENTRY(_int4_handler)
	SAVE_ALL_INT
	CALL_INT 4
ENDPROC(_int4_handler)

ENTRY(_int5_handler)
	SAVE_ALL_INT
	CALL_INT 5
ENDPROC(_int5_handler)

ENTRY(_int6_handler)
	SAVE_ALL_INT
	CALL_INT 6
ENDPROC(_int6_handler)

ENTRY(_int7_handler)
	SAVE_ALL_INT
	CALL_INT 7
ENDPROC(_int7_handler)

ENTRY(_int8_handler)
	SAVE_ALL_INT
	CALL_INT 8
ENDPROC(_int8_handler)

ENTRY(_int9_handler)
	SAVE_ALL_INT
	CALL_INT 9
ENDPROC(_int9_handler)

ENTRY(_int10_handler)
	SAVE_ALL_INT
	CALL_INT 10
ENDPROC(_int10_handler)

ENTRY(_int11_handler)
	SAVE_ALL_INT
	CALL_INT 11
ENDPROC(_int11_handler)

ENTRY(_int12_handler)
	SAVE_ALL_INT
	CALL_INT 12
ENDPROC(_int12_handler)

ENTRY(_int13_handler)
	SAVE_ALL_INT
	CALL_INT 13
ENDPROC(_int13_handler)

ENTRY(_int14_handler)
	SAVE_ALL_INT
	CALL_INT 14
ENDPROC(_int14_handler)

ENTRY(_int15_handler)
	SAVE_ALL_INT
	CALL_INT 15
ENDPROC(_int15_handler)

	;;
	;; Handler for uninitialized and spurious interrupts
	;;
ENTRY(_bad_interrupt)
	B	.S2	IRP
	NOP	5
ENDPROC(_bad_interrupt)

	;;
	;; Entry for NMI/exceptions/syscall
	;;
ENTRY(_nmi_handler)
	SAVE_ALL NRP,NTSR

	MVC	.S2	EFR,B2
	CMPEQ	.L2	1,B2,B2
 ||	MVC	.S2	TSR,B1
	CLR	.S2	B1,10,10,B1
	MVC	.S2	B1,TSR
 [!B2]	B	.S2	process_exception
 [B2]	B	.S2	system_call_saved
 	ADDAW	.D2	SP,2,B1
 	MV	.D1X	B1,A4
	ADDKPC	.S2	ret_from_trap,B3,3

ret_from_trap:
	BNOP	.S2	ret_from_exception,5
ENDPROC(_nmi_handler)

ENTRY(enable_exception)
	DINT
	MVC	.S2	TSR,B0
	MVC	.S2	B3,NRP
	MVK	.L2	0xc,B1
	OR	.D2	B0,B1,B0
	MVC	.S2	B0,TSR			;  Set GEE and XEN in TSR
	B	.S2	NRP
	NOP	5
ENDPROC(enable_exception)
