/*
 * np-dsp - init.c
 *
 * Copyright (C) 2012, 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <kernel.h>
#include <compiler.h>
#include <pu-shm.h>
#include <asm_io.h>
#include <special_insns.h>
#include <timex.h>

#define CACHE_REGS	(u8 *) 0x01840000
#define CACHE_L2CFG	0x0000

#define CACHE_L1PCFG	0x0020
#define CACHE_L1P_MODE_4K	1
#define CACHE_L1P_MODE_8K	2
#define CACHE_L1P_MODE_16K	3
#define CACHE_L1P_MODE_32K	4

#define CACHE_L1DCFG	0x0040
#define CACHE_L1D_MODE_4K	1
#define CACHE_L1D_MODE_8K	2
#define CACHE_L1D_MODE_16K	3
#define CACHE_L1D_MODE_32K	4

#define CACHE_L2_MODE_32K	1
#define CACHE_L2_MODE_64K	2
#define CACHE_L2_MODE_128K	3
#define CACHE_L2_MODE_256K	4

static void tsc_start(void)
{
	set_creg(TSCL, 0);
}

static void enable_cache(void)
{
	writel(CACHE_L1P_MODE_32K, CACHE_REGS + CACHE_L1PCFG);
	readl(CACHE_REGS + CACHE_L1PCFG);

	writel(CACHE_L1D_MODE_16K, CACHE_REGS + CACHE_L1DCFG);
	readl(CACHE_REGS + CACHE_L1DCFG);

#if 0
	writel(CACHE_L2_MODE_128K, CACHE_REGS + CACHE_L2CFG);
	readl(CACHE_REGS + CACHE_L2CFG);
#endif
}
	
static void start_payload(void)
{
	__asm__ __volatile__(
			"B	.S2	payload_data\n"
			"NOP	5\n"
			);
}

void pu_shm_init(void);

void start_kernel(void)
{
	enable_cache();

	trap_init();
	printk_init();
	printk("Starting np-dsp...\n");
	irq_init();
	edma_init();
	mcasp_module_init();
	tsc_start();
	pu_shm_init();

	start_payload();
}
