/*
 * jafet - printk.c
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <asm_io.h>

#define PRINTK_BUF_MEM		0xc3f00000UL
#define PRINTK_BUF_SIZE		0x00040000	/* 256 KiB */

static char *printk_buf = (void *) PRINTK_BUF_MEM;
static char *printk_pos = (void *) PRINTK_BUF_MEM;

static void printk_puts(const char *s)
{
	char *pos;

	pos = printk_pos;
	while (*s) {
		*pos++ = *s++;
		if (pos == printk_buf + PRINTK_BUF_SIZE)
			pos = printk_buf;
	}
	printk_pos = pos;
}

int printk(const char *fmt, ...)
{
	static char buf[1024];
	va_list args;
	int n;
	
	va_start(args, fmt);
	n = vsnprintf(buf, sizeof(buf), fmt, args);
	va_end(args);
	if (n < 0)
		goto out;

	printk_puts(buf);

out:
	return n;
}

void printk_init(void)
{
	memset(printk_buf, 0, PRINTK_BUF_SIZE);
}
