/*
 * jafet - pu-shm.c
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <errno.h>
#include <pu-shm.h>
#include <asm_io.h>

#define DSP_DATA_MEM		0xc2000000
#define DSP_DATA_SIZE		0x01f00000	/* 31 MiB */

#define SUBSAMPLE_SIZE		0x8
#define SUBSAMPLE_ALIGN		0x8
#define SAMPLE_SIZE		0x1000
#define SAMPLE_ALIGN		0x1000

#define SUBSAMPLES_PER_SAMPLE	(SAMPLE_ALIGN / SUBSAMPLE_ALIGN)

#define SAMPLES_PER_GROUP	480

#define RING_ENTRIES		(DSP_DATA_SIZE - DSP_RING_BASE) / SAMPLE_ALIGN

static u8 __iomem *shm = (u8 __iomem *) DSP_DATA_MEM;
static u32 shm_seq;
static u32 shm_gen;
static unsigned int shm_subsample;

static u8 *dsp_get_buffer(unsigned int seq, unsigned int subsample)
{
	unsigned long offset;

	seq %= RING_ENTRIES;
	offset = seq * SAMPLE_ALIGN + subsample * SUBSAMPLE_ALIGN;
	return shm + DSP_RING_BASE + offset;
}

int pu_shm_store(const void *buffer, size_t len)
{
	const u32 *b = buffer;
	u32 *dst = (u32 *) dsp_get_buffer(shm_seq, shm_subsample);
	unsigned int nwords;
	static unsigned int x;
	unsigned int i;

	if ((u8 *) dst >= shm + DSP_DATA_SIZE) {
		writel(0xdeaddead, shm + DSP_MAGIC);
		return -EINVAL;
	}

	if (len < SUBSAMPLE_SIZE)
		return -EINVAL;

	writel((unsigned long) dst, shm + 0x80);
	writel((unsigned long) b, shm + 0x88);
	writel(x++, shm + 0x8c);

	nwords = SUBSAMPLE_SIZE >> 2;
	for (i = 0; i < nwords; i++)
		__raw_writel(b[i], dst + i);
	shm_subsample++;
	if (shm_subsample >= SUBSAMPLES_PER_SAMPLE) {
		shm_subsample = 0;
		shm_seq++;
		writel(shm_seq, shm + DSP_SEQNUM);
	}
	return 0;
}

void pu_shm_init(void)
{
	writel(0xdeadbeef, shm + DSP_MAGIC);

	shm_gen = readl(shm + DSP_GENERATION);
	shm_gen &= ~1;
	shm_gen++;
	writel(shm_gen, shm + DSP_GENERATION);

	writel(0, shm + DSP_VERSION);
	memset(shm + DSP_GENERATION + 4, 0, 4096 - (DSP_GENERATION + 4));

	writel(RING_ENTRIES, shm + DSP_RING_ENTRIES);

	shm_seq = 0;
	writel(shm_seq, shm + DSP_SEQNUM);

	writel(SAMPLE_SIZE, shm + DSP_SAMPLE_SIZE);
	writel(SAMPLES_PER_GROUP, shm + DSP_SAMPLES_PER_GROUP);
	writel(DSP_DATA_SIZE, shm + DSP_SHM_SIZE);
	writel(0, shm + DSP_IGROUP);

	shm_gen++;
	writel(shm_gen, shm + DSP_GENERATION);

	writel(DSP_MAGIC_VALUE, shm + DSP_MAGIC);
}
